package stk

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Class Container.containerSource

A container widget is a widget which can contain other widgets. The type parameter is the type of the position of a widget in the container. This class must be inherited, as it is for example by container_list, which specialises this class by using a list to store children widgets.