package stk

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Class Datetime.calendarSource

inherit Stk.Box.box

Properties

method month : month
method set_month : ?delay:float -> ?propagate:bool -> month -> unit
method prev_month : unit

cal#prev_month changes the p_month property to previous month, if this property was set.

method next_month : unit

cal#next_month changes the p_month property to next month, if this property was set.

method editable : bool
method set_editable : ?delay:float -> ?propagate:bool -> bool -> unit
method selected_dates : date list
method selection_mode : Stk.Props.selection_mode
method set_selection_mode : ?delay:float -> ?propagate:bool -> Stk.Props.selection_mode -> unit

Hacking

method clear : unit

cal#clear removes all day widgets.

method set_title : string -> unit

cal#set_title str changes title of calendar. Beware the title is also set when the month changes.

method private set_title_now : month -> unit
method day_label : int -> Stk.Text.label option

cal#day_label n returns the label widget corresponding to the given month day (1..<last day of month>).

method day_button : int -> Stk.Button.button option

cal#day_button n returns the button corresponding to the given month day (1..<last day of month>).

method day_labels_of_weekday : weekday -> (int * Stk.Text.label) list

cal#day_labels_of_weekday wd returns the list of label widgets for the given weekday.

Selection

method select_date : ?only:bool -> (int * int * int) -> unit
method private unselect_date_ : date -> unit
method unselect_date : date -> unit
method unselect_all : unit