package stk

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Stk.LayersSource

Multi-layer widget.

Sourcemodule A = Dynarray

Properties

Sourcetype event_direction =
  1. | Upward
  2. | Downward

This type describe how SDL events are propagated through layers.

Sourceval string_of_event_direction : event_direction -> string
Sourceval event_direction_of_string : string -> event_direction
Sourcemodule TEvent_direction : sig ... end
Sourcemodule PEvent_direction : sig ... end
Sourceval event_direction : event_direction Props.prop

Property used to store event direction.

The layers widget

Sourceclass layers : ?classes:string list option -> ?name:string option -> ?props:Props.t option -> ?wdata: Widget.wdata option -> unit -> object ... end

A layers widget can handle several widget trees, allowing multiple layers of widgets. A layer is identified by its position. Bottom layer has position 0. A layer can have a root widget associated (see the pack method). Widgets (and so, layers) are rendered from bottom layer up to top layer, in this order.

Sourcetype Widget.widget_type +=
  1. | Layers of layers
Sourceval layers : ?name:string -> ?props:Props.t -> ?wdata:Widget.wdata -> ?event_direction:event_direction -> ?pack:(Widget.widget -> unit) -> unit -> layers

Convenient function to create a layers. See Widget arguments for other arguments.