package stk

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Stk.SelectionSource

Handling current selection.

Current selection is retrieved using get. For widgets handling user selection, when the user selects something, the widget will set the function called to retrieve the selection. unset removes the selection function.

Sourceval unset : unit -> unit
Sourceval set : (unit -> string option) -> unit
Sourceval get : unit -> string option